/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse, die ein Attribut ein- und auslesen kann, welches nur Werte aus einer vorgegebenen Liste annehmen kann
*/
function AttributeSet(
    attributeName,      // Attributname
    values,             // Erlaubte Attributwerte der Form [ { key: "Attributwert", value: "Interner Wert" }, … ]
    casesensitive,      // Optional: Wird zwischen Groß- und Kleinschreibung unterschieden?
    required            // Optional: Muss dieses Attribut angegeben sein?
) {
    Object.defineProperty(this, 'attributeName', { 'value': attributeName, 'enumerable': true });
    this.values = (values instanceof Array) ? values.map(function(item) {
        if ((typeof item != 'object') || (item instanceof String) || (item instanceof Number)) {
            return { 'key': '' + item, 'value': item };
        }
        return item;
    }).filter(function(item) { return (item instanceof Object) && ((typeof item.key == 'string') || (item.key instanceof String)); }) : [];
    if (this.values.length)
        this.defaultValue = this.values[0].value;
    if (casesensitive)
        this.casesensitive = true;
    if (required)
        this.required = true;
}

// Attributname
AttributeSet.prototype.attributeName = 'auswahl';

// Erlaubte Attributwerte
AttributeSet.prototype.values = [];

// Wird zwischen Groß- und Kleinschreibung unterschieden?
AttributeSet.prototype.casesensitive = false;

// Muss dieses Attribut angegeben sein?
AttributeSet.prototype.required = false;

// Standardwert
AttributeSet.prototype.defaultValue = '';

/**
* Zum zugehörigen Attributwert in obj die Bezeichnung ermitteln
* @return String        Bezeichnung des Werts (false, wenn kein Attributwert angegeben)
*/
AttributeSet.prototype.nameValue = function(
    obj     // Objekt mit den Attributwerten
) {
    var value = obj[this.attributeName];
    if (value === undefined)
        return false;
    for (var n = 0; n < this.values.length; ++n) {
        if (this.values[n].value === value)
            return this.values[n].key;
    }
    return this.defaultValue;
}

/**
* Den zur Angabe strKey passenden Attributwert im Objekt obj eintragen
* @return String        Im Fehlerfall Fehlermeldung
*/
AttributeSet.prototype.readValue = function(
    strKey,     // Von Benutzer eingegebener Wert, der tatsächlichen Attributwert beschreibt
    obj         // Objekt für tatsächliche Attributwerte
) {
    if (!(typeof strKey == 'string') && !(strKey instanceof String)) {
        if (this.required)
        return 'Wert ' + (this.attributeName ? ('des Attributs „' + this.attributeName + '“ ') : '') + 'fehlt';
        obj[this.attributeName] = this.defaultValue;
        return;
    }

    var cleanedKey = this.casesensitive ? strKey : strKey.toLowerCase();
    var index = this.values.map(this.casesensitive ? function(item) { return item.key; } : function(item) { return item.key.toLowerCase(); }).indexOf(cleanedKey);

    if (index != -1) {
        obj[this.attributeName] = this.values[index].value;
        return;
    }
    return 'Ungültiger Wert „' + strKey + (this.attributeName ? ('“ für Attribut „' + this.attributeName) : '') + '“ (Erlaubte Werte: ' + this.values.map(function(item) { return item.key; } ).join(', ') + ')';
}
